from .Xitroo import Xitroo
from .exceptions import *

__title__ = "Unofficial Xitroo API"
__description__ = "Xitroo library for Python"
__url__ = "https://github.com/Th3K1n91/xitroo_api"
__version__ = "0.8"
__author__ = "InSuckaBlyat88"
__author_email__ = "x.xxha001@gmail.com"


